/**
  * elweb v0.8.0
  * (c) 2019 Abdelrahman Awad
  * @license MIT
  */
(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@vue/composition-api')) :
    typeof define === 'function' && define.amd ? define(['exports', '@vue/composition-api'], factory) :
    (global = global || self, factory(global.VueUseWeb = {}, global.vueCompositionApi));
}(this, (function (exports, compositionApi) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

    var __assign = function() {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };

    function __awaiter(thisArg, _arguments, P, generator) {
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
            function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
            function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }

    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f) throw new TypeError("Generator is already executing.");
            while (_) try {
                if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
                if (y = 0, t) op = [op[0] & 2, t.value];
                switch (op[0]) {
                    case 0: case 1: t = op; break;
                    case 4: _.label++; return { value: op[1], done: false };
                    case 5: _.label++; y = op[1]; op = [0]; continue;
                    case 7: op = _.ops.pop(); _.trys.pop(); continue;
                    default:
                        if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                        if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                        if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                        if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                        if (t[2]) _.ops.pop();
                        _.trys.pop(); continue;
                }
                op = body.call(thisArg, _);
            } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
            if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
        }
    }

    var events = ['chargingchange', 'chargingtimechange', 'dischargingtimechange', 'levelchange'];
    function useBattery() {
        var state = compositionApi.reactive({
            isCharging: false,
            chargingTime: 0,
            dischargingTime: 0,
            level: 1
        });
        function updateBatteryInfo() {
            state.isCharging = this.charging;
            state.chargingTime = this.chargingTime || 0;
            state.dischargingTime = this.dischargingTime || 0;
            state.level = this.level;
        }
        compositionApi.onMounted(function () {
            if (!('getBattery' in navigator)) {
                return;
            }
            navigator.getBattery().then(function (battery) {
                updateBatteryInfo.call(battery);
                events.forEach(function (evt) {
                    battery.addEventListener(evt, updateBatteryInfo);
                });
            });
        });
        compositionApi.onUnmounted(function () {
            if (!('getBattery' in navigator)) {
                return;
            }
            navigator.getBattery().then(function (battery) {
                events.forEach(function (evt) {
                    battery.removeEventListener(evt, updateBatteryInfo);
                });
            });
        });
        return __assign({}, compositionApi.toRefs(state));
    }

    function useClipboard() {
        var text = compositionApi.ref('');
        function onCopy() {
            return __awaiter(this, void 0, void 0, function () {
                var _a;
                return __generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            _a = text;
                            return [4 /*yield*/, navigator.clipboard.readText()];
                        case 1:
                            _a.value = _b.sent();
                            return [2 /*return*/];
                    }
                });
            });
        }
        compositionApi.onMounted(function () {
            window.addEventListener('copy', onCopy);
        });
        compositionApi.onUnmounted(function () {
            window.removeEventListener('copy', onCopy);
        });
        function write(txt) {
            text.value = txt;
            return navigator.clipboard.writeText(txt);
        }
        return {
            text: text,
            write: write
        };
    }

    function useDeviceLight() {
        var value = compositionApi.ref(null);
        function handler(event) {
            value.value = event.value;
        }
        // TODO: Should we debounce/throttle the event?
        compositionApi.onMounted(function () {
            window.addEventListener('devicelight', handler);
        });
        compositionApi.onUnmounted(function () {
            window.removeEventListener('devicelight', handler);
        });
        return {
            value: value
        };
    }

    function throttle(wait, fn) {
        if (wait === 0) {
            return fn;
        }
        var timeout;
        return (function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var later = function () {
                timeout = undefined;
                // check if the fn call was cancelled.
                fn.apply(void 0, args);
            };
            clearTimeout(timeout);
            timeout = setTimeout(later, wait);
        });
    }
    var hasWindow = typeof window !== 'undefined';
    function useEventListener(element, type, listener, options) {
        compositionApi.onMounted(function () {
            element.addEventListener(type, listener, options);
        });
        compositionApi.onUnmounted(function () {
            element.removeEventListener(type, listener, options);
        });
    }
    var eventListenerFor = function (element) { return function (type, listener, options) { return useEventListener(element, type, listener, options); }; };
    var useDocumentEventListener = eventListenerFor(document);

    function initState() {
        var defaults = function () { return ({
            x: null,
            y: null,
            z: null
        }); };
        var data = {
            acceleration: defaults(),
            accelerationIncludingGravity: defaults(),
            rotationRate: {
                alpha: null,
                beta: null,
                gamma: null
            },
            interval: 0
        };
        return compositionApi.reactive(data);
    }
    function useDeviceMotion(options) {
        if (options === void 0) { options = { throttleMs: 10 }; }
        var state = initState();
        function onDeviceMotion(event) {
            state.acceleration = event.acceleration;
            state.accelerationIncludingGravity = event.accelerationIncludingGravity;
            state.rotationRate = event.rotationRate;
            state.interval = event.interval;
        }
        var handler = options.throttleMs ? throttle(options.throttleMs, onDeviceMotion) : onDeviceMotion;
        compositionApi.onMounted(function () {
            window.addEventListener('devicemotion', handler, false);
        });
        compositionApi.onUnmounted(function () {
            window.removeEventListener('devicemotion', handler, false);
        });
        return __assign({}, compositionApi.toRefs(state));
    }

    function initState$1() {
        var data = {
            isAbsolute: false,
            gamma: 0,
            alpha: 0,
            beta: 0
        };
        return compositionApi.reactive(data);
    }
    function useDeviceOrientation() {
        var state = initState$1();
        function handler(event) {
            state.isAbsolute = event.absolute;
            state.alpha = event.alpha;
            state.beta = event.beta;
            state.gamma = event.gamma;
        }
        compositionApi.onMounted(function () {
            window.addEventListener('deviceorientation', handler);
        });
        compositionApi.onUnmounted(function () {
            window.removeEventListener('deviceorientation', handler);
        });
        return __assign({}, compositionApi.toRefs(state));
    }

    /**
     * Tracks visibility of the page.
     *
     * @returns True if the document is currently visible.
     *
     * @example
     * setup() {
     *  const isVisible = useDocumentVisibility()
     *  watch(() => console.log(isVisible))
     * }
     *
     */
    function useDocumentVisibility() {
        var documentIsVisible = function () { return document.visibilityState === 'visible'; };
        var isVisible = compositionApi.ref(hasWindow ? documentIsVisible() : true);
        useDocumentEventListener('visibilitychange', function () {
            isVisible.value = documentIsVisible();
        });
        return isVisible;
    }

    function useFetch(url, options) {
        var stateDefs = {
            isLoading: false,
            success: false,
            error: false,
            cancelled: false,
            json: null,
            blob: null,
            text: '',
            type: 'unknown',
            status: undefined,
            statusText: '',
            headers: {}
        };
        var state = compositionApi.reactive(stateDefs);
        var signal;
        var controller;
        if (typeof AbortController !== 'undefined') {
            controller = new AbortController();
            signal = controller.signal;
        }
        function execute() {
            state.isLoading = true;
            return window
                .fetch(url, __assign({ signal: signal }, options))
                .then(function (res) {
                state.success = res.ok;
                state.error = !res.ok;
                state.isLoading = false;
                state.statusText = res.statusText;
                state.status = res.status;
                state.type = res.type;
                var headers = {};
                res.headers.forEach(function (value, key) {
                    headers[key] = value;
                });
                state.headers = headers;
                return Promise.all([res.clone().text(), res.clone().blob(), res.json()]);
            })
                .then(function (_a) {
                var text = _a[0], blob = _a[1], json = _a[2];
                // Graceful degradation for cancellation.
                if (state.cancelled) {
                    return;
                }
                state.text = text;
                state.blob = blob;
                state.json = json;
            })
                .catch(function () {
                state.error = true;
                state.isLoading = false;
            });
        }
        compositionApi.onMounted(execute);
        function cancel() {
            if (state.success) {
                return;
            }
            state.cancelled = true;
            if (controller) {
                controller.abort();
            }
        }
        return __assign(__assign({}, compositionApi.toRefs(state)), { cancel: cancel,
            execute: execute });
    }

    function useFullscreen(target) {
        var isFullscreen = compositionApi.ref(false);
        function exitFullscreen() {
            if (document.fullscreenElement) {
                document.exitFullscreen();
            }
            isFullscreen.value = false;
        }
        function enterFullscreen() {
            exitFullscreen();
            target.value.requestFullscreen().then(function () {
                isFullscreen.value = true;
            });
        }
        return {
            isFullscreen: isFullscreen,
            enterFullscreen: enterFullscreen,
            exitFullscreen: exitFullscreen
        };
    }

    function useGeolocation(options) {
        var data = {
            coords: {
                accuracy: 0,
                latitude: 0,
                longitude: 0,
                altitude: null,
                altitudeAccuracy: null,
                heading: null,
                speed: null
            },
            locatedAt: undefined,
            error: ''
        };
        var state = compositionApi.reactive(data);
        function updatePosition(position) {
            state.locatedAt = new Date(position.timestamp);
            state.coords = position.coords;
            state.error = '';
        }
        var watcher;
        compositionApi.onMounted(function () {
            if ('geolocation' in navigator) {
                watcher = window.navigator.geolocation.watchPosition(updatePosition, undefined, options);
            }
        });
        compositionApi.onUnmounted(function () {
            if (watcher) {
                window.navigator.geolocation.clearWatch(watcher);
            }
        });
        return __assign({}, compositionApi.toRefs(state));
    }

    function useIntersectionObserver(target, options) {
        if (options === void 0) { options = { root: null, rootMargin: '0px' }; }
        var state = compositionApi.reactive({
            intersectionRatio: 0,
            isIntersecting: false,
            isFullyInView: false
        });
        function observe() {
            if (target.value) {
                observer.observe(target.value);
            }
        }
        var observer;
        compositionApi.onMounted(function () {
            observer = new IntersectionObserver(function (_a) {
                var entry = _a[0];
                state.intersectionRatio = entry.intersectionRatio;
                if (entry.intersectionRatio > 0) {
                    state.isIntersecting = true;
                    state.isFullyInView = entry.intersectionRatio >= 1;
                    return;
                }
                state.isIntersecting = false;
            }, options);
            observe();
        });
        function unobserve() {
            if (!observer)
                return;
            if (target.value) {
                observer.unobserve(target.value);
            }
        }
        compositionApi.onUnmounted(unobserve);
        return __assign(__assign({}, compositionApi.toRefs(state)), { observe: observe,
            unobserve: unobserve });
    }

    function parseValue(serializedVal) {
        var value = null;
        try {
            value = JSON.parse(serializedVal);
        }
        catch (_a) {
            value = serializedVal;
        }
        return value;
    }
    function useLocalStorage(key, def) {
        if (def === void 0) { def = undefined; }
        var value = compositionApi.ref(null);
        var init = function () {
            var serializedVal = localStorage.getItem(key);
            if (serializedVal !== null) {
                value.value = parseValue(serializedVal);
                return;
            }
            value.value = def;
        };
        var initialized = false;
        // early init if possible.
        if (typeof window !== 'undefined') {
            init();
            initialized = true;
        }
        function handler(e) {
            if (e.key === key) {
                value.value = e.newValue ? parseValue(e.newValue) : null;
            }
        }
        compositionApi.onMounted(function () {
            if (!initialized) {
                init();
            }
            window.addEventListener('storage', handler, true);
        });
        compositionApi.onUnmounted(function () {
            localStorage.setItem(key, JSON.stringify(value.value));
            window.removeEventListener('storage', handler);
        });
        return {
            value: value
        };
    }

    function useMedia(query) {
        var mediaQuery;
        // try to fetch initial value (avoid SSR issues)
        if (typeof window !== 'undefined') {
            mediaQuery = window.matchMedia(query);
        }
        var matches = compositionApi.ref(mediaQuery ? mediaQuery.matches : false);
        function handler(event) {
            matches.value = event.matches;
        }
        compositionApi.onMounted(function () {
            if (!mediaQuery) {
                mediaQuery = window.matchMedia(query);
            }
            matches.value = mediaQuery.matches;
            mediaQuery.addListener(handler);
        });
        compositionApi.onUnmounted(function () {
            mediaQuery.removeListener(handler);
        });
        return matches;
    }

    function useMousePosition() {
        var state = compositionApi.reactive({
            x: 0,
            y: 0
        });
        function handler(e) {
            state.x = e.clientX;
            state.y = e.clientY;
        }
        compositionApi.onMounted(function () {
            window.addEventListener('mousemove', handler, false);
        });
        compositionApi.onUnmounted(function () {
            window.removeEventListener('mousemove', handler, false);
        });
        return __assign({}, compositionApi.toRefs(state));
    }

    function makeState() {
        var data = {
            isOnline: true,
            offlineAt: undefined,
            downlink: undefined,
            downlinkMax: undefined,
            effectiveType: undefined,
            saveData: undefined,
            type: undefined
        };
        return compositionApi.reactive(data);
    }
    function useNetwork() {
        var state = makeState();
        function updateNetworkInformation() {
            state.isOnline = window.navigator.onLine;
            state.offlineAt = state.isOnline ? undefined : new Date();
            // skip for non supported browsers.
            if (!('connection' in window.navigator)) {
                return;
            }
            state.downlink = window.navigator.connection.downlink;
            state.downlinkMax = window.navigator.connection.downlinkMax;
            state.effectiveType = window.navigator.connection.effectiveType;
            state.saveData = window.navigator.connection.saveData;
            state.type = window.navigator.connection.type;
        }
        var onOffline = function () {
            state.isOnline = false;
            state.offlineAt = new Date();
        };
        var onOnline = function () {
            state.isOnline = true;
        };
        compositionApi.onMounted(function () {
            updateNetworkInformation();
            window.addEventListener('offline', onOffline);
            window.addEventListener('online', onOnline);
            if ('connection' in window.navigator) {
                window.navigator.connection.addEventListener('change', updateNetworkInformation, false);
            }
        });
        compositionApi.onUnmounted(function () {
            window.removeEventListener('offline', onOffline);
            window.removeEventListener('online', onOnline);
            if ('connection' in window.navigator) {
                window.navigator.connection.removeEventListener('change', updateNetworkInformation, false);
            }
        });
        return __assign({}, compositionApi.toRefs(state));
    }

    function usePreferredColorScheme() {
        var queries = {
            light: '(prefers-color-scheme: light)',
            dark: '(prefers-color-scheme: dark)',
            'no-preference': '(prefers-color-scheme: no-preference)'
        };
        var isDark = useMedia(queries.dark);
        var isLight = useMedia(queries.light);
        var theme = compositionApi.computed(function () {
            if (isDark.value) {
                return 'dark';
            }
            if (isLight.value) {
                return 'light';
            }
            return 'no-preference';
        });
        return theme;
    }

    function usePreferredLanguages() {
        var value = compositionApi.ref(navigator.languages);
        function handler() {
            value.value = navigator.languages;
        }
        compositionApi.onMounted(function () {
            window.addEventListener('languagechange', handler);
        });
        compositionApi.onUnmounted(function () {
            window.removeEventListener('languagechange', handler);
        });
        return value;
    }

    function useScript(opts) {
        var state = compositionApi.reactive({
            isLoading: false,
            error: false,
            success: false
        });
        var promise = new Promise(function (resolve, reject) {
            compositionApi.onMounted(function () {
                var script = document.createElement('script');
                // script.async = opts.async || true;
                // script.async = opts.defer || true;
                // script.noModule = !opts.module || false;
                script.onload = function () {
                    state.isLoading = false;
                    state.success = true;
                    state.error = false;
                    resolve();
                };
                script.onerror = function (err) {
                    state.isLoading = false;
                    state.success = false;
                    state.error = true;
                    reject(err);
                };
                script.src = opts.src;
                document.head.appendChild(script);
            });
        });
        return __assign(__assign({}, compositionApi.toRefs(state)), { toPromise: function () { return promise; } });
    }

    function useWebSocket(url) {
        var data = compositionApi.ref(null);
        var state = compositionApi.ref('CONNECTING');
        var ws;
        var close = function close(code, reason) {
            if (!ws)
                return;
            ws.close(code, reason);
        };
        var send = function send(data) {
            if (!ws)
                return;
            ws.send(data);
        };
        compositionApi.onMounted(function () {
            ws = new WebSocket(url);
            ws.onopen = function () {
                state.value = 'OPEN';
            };
            ws.onclose = ws.onerror = function () {
                state.value = 'CLOSED';
            };
            ws.onmessage = function (e) {
                data.value = e.data;
            };
        });
        compositionApi.onUnmounted(function () {
            ws.close();
        });
        return {
            data: data,
            state: state,
            close: close,
            send: send
        };
    }

    function useWindowScrollPosition(options) {
        if (options === void 0) { options = { throttleMs: 100 }; }
        var state = compositionApi.reactive({ x: 0, y: 0 });
        function setScrollPos() {
            state.x = window.pageXOffset;
            state.y = window.pageYOffset;
        }
        var onScroll = throttle(options.throttleMs, setScrollPos);
        compositionApi.onBeforeMount(function () {
            setScrollPos();
        });
        compositionApi.onMounted(function () {
            window.addEventListener('scroll', onScroll, { passive: true });
        });
        compositionApi.onUnmounted(function () {
            window.removeEventListener('scroll', onScroll);
        });
        return __assign({}, compositionApi.toRefs(state));
    }

    function useWindowSize(options) {
        if (options === void 0) { options = { throttleMs: 100 }; }
        var state = compositionApi.reactive({ width: 0, height: 0 });
        function setSize() {
            state.width = window.innerWidth;
            state.height = window.innerHeight;
        }
        var onScroll = throttle(options.throttleMs, setSize);
        compositionApi.onBeforeMount(function () {
            setSize();
        });
        compositionApi.onMounted(function () {
            window.addEventListener('resize', onScroll, { passive: true });
        });
        compositionApi.onUnmounted(function () {
            window.removeEventListener('resize', onScroll);
        });
        return __assign({}, compositionApi.toRefs(state));
    }

    exports.useBattery = useBattery;
    exports.useClipboard = useClipboard;
    exports.useDeviceLight = useDeviceLight;
    exports.useDeviceMotion = useDeviceMotion;
    exports.useDeviceOrientation = useDeviceOrientation;
    exports.useDocumentVisibility = useDocumentVisibility;
    exports.useFetch = useFetch;
    exports.useFullscreen = useFullscreen;
    exports.useGeolocation = useGeolocation;
    exports.useIntersectionObserver = useIntersectionObserver;
    exports.useLocalStorage = useLocalStorage;
    exports.useMedia = useMedia;
    exports.useMousePosition = useMousePosition;
    exports.useNetwork = useNetwork;
    exports.usePreferredColorScheme = usePreferredColorScheme;
    exports.usePreferredLanguages = usePreferredLanguages;
    exports.useScript = useScript;
    exports.useWebSocket = useWebSocket;
    exports.useWindowScrollPosition = useWindowScrollPosition;
    exports.useWindowSize = useWindowSize;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
